'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var backendCommon = require('@backstage/backend-common');
var express = require('express');
var Router = require('express-promise-router');
var utilArnParser = require('@aws-sdk/util-arn-parser');
var awsCorePluginForBackstageCommon = require('@aws/aws-core-plugin-for-backstage-common');
var integrationAwsNode = require('@backstage/integration-aws-node');
var clientCodepipeline = require('@aws-sdk/client-codepipeline');
var awsCodepipelinePluginForBackstageCommon = require('@aws/aws-codepipeline-plugin-for-backstage-common');
var backendPluginApi = require('@backstage/backend-plugin-api');
var alpha = require('@backstage/plugin-catalog-node/alpha');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e : { default: e }; }

var express__default = /*#__PURE__*/_interopDefaultCompat(express);
var Router__default = /*#__PURE__*/_interopDefaultCompat(Router);

const DEFAULT_EXECUTIONS_LIMIT = 100;
class DefaultAwsCodePipelineService {
  constructor(logger, catalogApi, resourceLocator, credsManager) {
    this.logger = logger;
    this.catalogApi = catalogApi;
    this.resourceLocator = resourceLocator;
    this.credsManager = credsManager;
  }
  static async fromConfig(config, options) {
    var _a;
    const credsManager = integrationAwsNode.DefaultAwsCredentialsManager.fromConfig(config);
    const resourceLocator = (_a = options == null ? void 0 : options.resourceLocator) != null ? _a : await awsCorePluginForBackstageCommon.AwsResourceLocatorFactory.fromConfig(config, options.logger);
    return new DefaultAwsCodePipelineService(
      options.logger,
      options.catalogApi,
      resourceLocator,
      credsManager
    );
  }
  async getPipelineExecutionsByEntity(entityRef) {
    var _a;
    (_a = this.logger) == null ? void 0 : _a.debug(`Fetch CodePipeline executions for ${entityRef}`);
    const arns = await this.getPipelineArnsForEntity(entityRef);
    const pipelineExecutions = await Promise.all(
      arns.map(async (arn) => {
        const { region, resource } = utilArnParser.parse(arn);
        const pipelineName = resource;
        const client = await this.getClient(region, arn);
        const paginatorConfig = {
          client,
          pageSize: 25
        };
        const commandParams = {
          pipelineName
        };
        const paginator = clientCodepipeline.paginateListPipelineExecutions(
          paginatorConfig,
          commandParams
        );
        const pipelineExecutionSummaries = [];
        for await (const page of paginator) {
          const executions = page.pipelineExecutionSummaries || [];
          if (pipelineExecutionSummaries.length + executions.length > DEFAULT_EXECUTIONS_LIMIT) {
            pipelineExecutionSummaries.push(
              ...executions.slice(
                0,
                DEFAULT_EXECUTIONS_LIMIT - pipelineExecutionSummaries.length
              )
            );
          } else {
            pipelineExecutionSummaries.push(...executions);
          }
          if (pipelineExecutionSummaries.length === DEFAULT_EXECUTIONS_LIMIT) {
            break;
          }
        }
        return {
          pipelineName,
          pipelineRegion: region,
          pipelineArn: arn,
          pipelineExecutions: pipelineExecutionSummaries
        };
      })
    );
    return {
      pipelineExecutions
    };
  }
  async getPipelineStateByEntity(entityRef) {
    var _a;
    (_a = this.logger) == null ? void 0 : _a.debug(`Fetch CodePipeline state for ${entityRef}`);
    const arns = await this.getPipelineArnsForEntity(entityRef);
    const pipelines = await Promise.all(
      arns.map(async (arn) => {
        const { region, resource } = utilArnParser.parse(arn);
        const pipelineName = resource;
        const client = await this.getClient(region, arn);
        const pipelineState = await client.send(
          new clientCodepipeline.GetPipelineStateCommand({
            name: pipelineName
          })
        );
        return {
          pipelineName,
          pipelineRegion: region,
          pipelineArn: arn,
          pipelineState
        };
      })
    );
    return {
      pipelines
    };
  }
  async getPipelineArnsForEntity(entityRef) {
    const entity = await this.catalogApi.getEntityByRef(entityRef);
    if (!entity) {
      throw new Error(`Failed to find entity ${JSON.stringify(entityRef)}`);
    }
    const annotation = awsCorePluginForBackstageCommon.getOneOfEntityAnnotations(entity, [
      awsCodepipelinePluginForBackstageCommon.AWS_CODEPIPELINE_ARN_ANNOTATION,
      awsCodepipelinePluginForBackstageCommon.AWS_CODEPIPELINE_TAGS_ANNOTATION
    ]);
    if (!annotation) {
      throw new Error("Annotation not found on entity");
    }
    let arns;
    if (annotation.name === awsCodepipelinePluginForBackstageCommon.AWS_CODEPIPELINE_TAGS_ANNOTATION) {
      arns = await this.resourceLocator.getResourceArns({
        resourceType: "AWS::CodePipeline::Pipeline",
        tagString: annotation.value
      });
    } else {
      arns = [annotation.value];
    }
    return Promise.resolve(arns);
  }
  async getClient(region, arn) {
    const credentialProvider = await this.credsManager.getCredentialProvider({
      arn
    });
    return new clientCodepipeline.CodePipelineClient({
      region,
      customUserAgent: "aws-codepipeline-plugin-for-backstage",
      credentialDefaultProvider: () => credentialProvider.sdkCredentialProvider
    });
  }
}

async function createRouter(options) {
  const { logger, awsCodePipelineApi } = options;
  const router = Router__default.default();
  router.use(express__default.default.json());
  router.get(
    "/v1/entity/:namespace/:kind/:name/executions",
    async (request, response) => {
      const { namespace, kind, name } = request.params;
      const services = await awsCodePipelineApi.getPipelineExecutionsByEntity({
        kind,
        namespace,
        name
      });
      response.status(200).json(services);
    }
  );
  router.get(
    "/v1/entity/:namespace/:kind/:name/state",
    async (request, response) => {
      const { namespace, kind, name } = request.params;
      const services = await awsCodePipelineApi.getPipelineStateByEntity({
        kind,
        namespace,
        name
      });
      response.status(200).json(services);
    }
  );
  router.get("/health", (_, response) => {
    logger.info("PONG!");
    response.json({ status: "ok" });
  });
  router.use(backendCommon.errorHandler());
  return router;
}

const awsCodePiplinePlugin = backendPluginApi.createBackendPlugin({
  pluginId: "aws-codepipeline",
  register(env) {
    env.registerInit({
      deps: {
        logger: backendPluginApi.coreServices.logger,
        httpRouter: backendPluginApi.coreServices.httpRouter,
        config: backendPluginApi.coreServices.rootConfig,
        catalogApi: alpha.catalogServiceRef
      },
      async init({ logger, httpRouter, config, catalogApi }) {
        const winstonLogger = backendCommon.loggerToWinstonLogger(logger);
        const awsCodePipelineApi = await DefaultAwsCodePipelineService.fromConfig(config, {
          catalogApi,
          logger: winstonLogger
        });
        httpRouter.use(
          await createRouter({
            logger: winstonLogger,
            awsCodePipelineApi
          })
        );
      }
    });
  }
});

exports.DefaultAwsCodePipelineService = DefaultAwsCodePipelineService;
exports.createRouter = createRouter;
exports.default = awsCodePiplinePlugin;
//# sourceMappingURL=index.cjs.js.map
